BEGIN TRANSACTION;
CREATE TABLE usuario_estructura (
  id INTEGER  NOT NULL  ,
  usuario_id INTEGER  NOT NULL  ,
  centro_fisico_id INTEGER    ,
  servicio_id INTEGER    ,
  equipo_id INTEGER    ,
  puesto_id INTEGER    ,
  observaciones VARCHAR(500)    ,
  activo VARCHAR(1)  NOT NULL    ,
PRIMARY KEY(id)    ,
  FOREIGN KEY(usuario_id)
    REFERENCES usuario(id)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(centro_fisico_id, servicio_id, equipo_id, puesto_id)
    REFERENCES estructura(centro_fisico_id, servicio_id, categoria_id, puesto_id)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION);
CREATE TABLE "usuario" (
	`id`	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	`persona_id`	INTEGER,
	`nick`	VARCHAR(8) NOT NULL UNIQUE,
	`passwd`	VARCHAR(8) NOT NULL,
	`fecha_alta`	DATE NOT NULL,
	`fecha_baja`	DATE,
	`intentos`	INTEGER NOT NULL DEFAULT 5,
	`activo`	VARCHAR(1) NOT NULL
);
CREATE TABLE "unit" (
	`id`	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	`tipo_unit_id`	INTEGER NOT NULL,
	`codigo`	VARCHAR(5) NOT NULL UNIQUE,
	`descripcion`	VARCHAR(255) NOT NULL UNIQUE,
	`activo`	VARCHAR(1) NOT NULL,
	`direccion`	VARCHAR(255),
	`poblacion`	VARCHAR(255),
	`cp`	VARCHAR(5),
	`provincia`	VARCHAR(50),
	`pais`	VARCHAR(50),
	`telefono1`	VARCHAR(20),
	`telefono2`	VARCHAR(20),
	`observaciones`	VARCHAR(500),
	`email`	VARCHAR(50),
	FOREIGN KEY(`tipo_unit_id`) REFERENCES `tipo_unit`(`id`) ON DELETE NO ACTION ON UPDATE NO ACTION
);
CREATE TABLE "turno_master" (
	`id`	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	`codigo`	VARCHAR(5) NOT NULL UNIQUE,
	`descripcion`	VARCHAR(255) NOT NULL UNIQUE,
	`cuenta_horas`	VARCHAR(1) NOT NULL,
	`activo`	VARCHAR(1) NOT NULL
);
CREATE TABLE turno_detail (
  turno_master_id INTEGER  NOT NULL  ,
  id INTEGER  NOT NULL  ,
  dia_inicial INTEGER  NOT NULL  ,
  dia_final INTEGER  NOT NULL  ,
  hora_inicio VARCHAR(5)  NOT NULL  ,
  hora_fin VARCHAR(5)  NOT NULL    ,
PRIMARY KEY(turno_master_id, id)  ,
  FOREIGN KEY(turno_master_id)
    REFERENCES turno_master(id)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION);
CREATE TABLE "tipo_unit" (
	`id`	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	`codigo`	VARCHAR(5) NOT NULL UNIQUE,
	`descripcion`	VARCHAR(255) NOT NULL,
	`activo`	VARCHAR(1) NOT NULL,
	`observaciones`	VARCHAR(255)
);
CREATE TABLE "tarea" (
	`id`	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	`puesto_id`	INTEGER NOT NULL,
	`fecha_inicio`	DATE NOT NULL,
	`fecha_fin`	DATE NOT NULL,
	`observaciones`	VARCHAR(500),
	`contrato_id`	INTEGER NOT NULL,
	`solapado`	VARCHAR(1) NOT NULL,
	`tarea_anterior`	INTEGER,
	`activo`	VARCHAR(1) NOT NULL,
	FOREIGN KEY(`puesto_id`) REFERENCES `unit`(`id`) ON DELETE NO ACTION ON UPDATE NO ACTION,
	FOREIGN KEY(`contrato_id`) REFERENCES `contrato`(`id`) ON DELETE NO ACTION ON UPDATE NO ACTION
);
CREATE TABLE "servicios_previos" (
	`id`	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	`persona_id`	INTEGER NOT NULL,
	`anno`	INTEGER NOT NULL,
	`horas`	INTEGER NOT NULL,
	FOREIGN KEY(`persona_id`) REFERENCES `persona`(`id`) ON DELETE NO ACTION ON UPDATE NO ACTION
);
CREATE TABLE "rol_usuario" (
	`id`	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	`rol_id`	INTEGER NOT NULL,
	`usuario_id`	INTEGER NOT NULL,
	`observaciones`	VARCHAR(500),
	`activo`	VARCHAR(1) NOT NULL,
	FOREIGN KEY(`rol_id`) REFERENCES `rol`(`id`) ON DELETE NO ACTION ON UPDATE NO ACTION,
	FOREIGN KEY(`usuario_id`) REFERENCES `usuario`(`id`) ON DELETE NO ACTION ON UPDATE NO ACTION
);
CREATE TABLE "rol_recurso" (
	`id`	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	`rol_id`	INTEGER NOT NULL,
	`recurso_id`	INTEGER NOT NULL,
	`ejecucion`	VARCHAR(1) NOT NULL,
	`lectura`	VARCHAR(1) NOT NULL,
	`escritura`	VARCHAR(1) NOT NULL,
	`observaciones`	VARCHAR(500),
	`activo`	VARCHAR(1) NOT NULL,
	FOREIGN KEY(`rol_id`) REFERENCES `rol`(`id`) ON DELETE NO ACTION ON UPDATE NO ACTION,
	FOREIGN KEY(`recurso_id`) REFERENCES `recurso`(`id`) ON DELETE NO ACTION ON UPDATE NO ACTION
);
CREATE TABLE "rol" (
	`id`	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	`codigo`	VARCHAR(5) NOT NULL UNIQUE,
	`descripcion`	VARCHAR(255) NOT NULL UNIQUE,
	`observaciones`	VARCHAR(500),
	`activo`	VARCHAR(1) NOT NULL
);
CREATE TABLE "recurso" (
	`id`	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	`codigo`	VARCHAR(5) NOT NULL UNIQUE,
	`descripcion`	VARCHAR(255) NOT NULL UNIQUE,
	`activo`	VARCHAR(1) NOT NULL,
	`observaciones`	VARCHAR(500)
);
CREATE TABLE "puesto_ciclo" (
	`id`	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	`ciclo_master_id`	INTEGER NOT NULL,
	`fecha_inicio`	DATE NOT NULL,
	`puesto_id`	INTEGER NOT NULL,
	`observaciones`	VARCHAR(500),
	`fecha_fin`	DATE NOT NULL,
	FOREIGN KEY(`ciclo_master_id`) REFERENCES `ciclo_master`(`id`) ON DELETE NO ACTION ON UPDATE NO ACTION,
	FOREIGN KEY(`puesto_id`) REFERENCES `unit`(`id`) ON DELETE NO ACTION ON UPDATE NO ACTION
);
CREATE TABLE "planificacion" (
	`mes`	INTEGER NOT NULL,
	`anno`	INTEGER NOT NULL,
	`puesto_id`	INTEGER NOT NULL,
	`fecha_inicio`	DATE NOT NULL,
	`ciclo_master_id`	INTEGER NOT NULL,
	`fecha_fin`	DATE NOT NULL,
	`total_dias`	INTEGER NOT NULL,
	`semana`	INTEGER NOT NULL,
	`dia1`	VARCHAR(5),
	`dia2`	VARCHAR(5),
	`dia3`	VARCHAR(5),
	`dia4`	VARCHAR(5),
	`dia5`	VARCHAR(5),
	`dia6`	VARCHAR(5),
	`dia7`	VARCHAR(5),
	`dia8`	VARCHAR(5),
	`dia9`	VARCHAR(5),
	`dia10`	VARCHAR(5),
	`dia11`	VARCHAR(5),
	`dia12`	VARCHAR(5),
	`dia13`	VARCHAR(5),
	`dia14`	VARCHAR(5),
	`dia15`	VARCHAR(5),
	`dia16`	VARCHAR(5),
	`dia17`	VARCHAR(5),
	`dia18`	VARCHAR(5),
	`dia19`	VARCHAR(5),
	`dia20`	VARCHAR(5),
	`dia21`	VARCHAR(5),
	`dia22`	VARCHAR(5),
	`dia23`	VARCHAR(5),
	`dia24`	VARCHAR(5),
	`dia25`	VARCHAR(5),
	`dia26`	VARCHAR(5),
	`dia27`	VARCHAR(5),
	`dia28`	VARCHAR(5),
	`dia29`	VARCHAR(5),
	`dia30`	VARCHAR(5),
	`dia31`	VARCHAR(5),
	`es_lunes_festivo`	VARCHAR(1) NOT NULL,
	`es_martes_festivo`	VARCHAR(1) NOT NULL,
	`es_miercoles_festivo`	VARCHAR(1) NOT NULL,
	`es_jueves_festivo`	VARCHAR(1) NOT NULL,
	`es_viernes_festivo`	VARCHAR(1) NOT NULL,
	`es_sabado_festivo`	VARCHAR(1) NOT NULL,
	`es_domingo_festivo`	VARCHAR(1) NOT NULL,
	`turno_libre_id`	INTEGER NOT NULL,
	PRIMARY KEY(`mes`,`anno`,`puesto_id`,`fecha_inicio`)
);
CREATE TABLE "persona" (
	`id`	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	`dni`	VARCHAR(9) NOT NULL UNIQUE,
	`nombre`	VARCHAR(100) NOT NULL,
	`ape1`	VARCHAR(100) NOT NULL,
	`ape2`	VARCHAR(100),
	`direccion`	VARCHAR(100),
	`cp`	VARCHAR(5),
	`poblacion`	VARCHAR(100),
	`provincia`	VARCHAR(50),
	`pais`	VARCHAR(50),
	`tlfno1`	VARCHAR(20) NOT NULL,
	`tlfno2`	VARCHAR(20),
	`email`	VARCHAR(50) NOT NULL,
	`observaciones`	VARCHAR(500),
	`sexo`	VARCHAR(1),
	`fnac`	DATE
);
CREATE TABLE jornada_teorica (
  centro_fisico_id INTEGER  NOT NULL  ,
  anno INTEGER  NOT NULL  ,
  total_horas_anual INTEGER    ,
  observaciones VARCHAR(500)      ,
PRIMARY KEY(centro_fisico_id, anno)  ,
  FOREIGN KEY(centro_fisico_id)
    REFERENCES unit(id)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION);
CREATE TABLE estructura (
  centro_fisico_id INTEGER  NOT NULL  ,
  servicio_id INTEGER  NOT NULL  ,
  categoria_id INTEGER  NOT NULL  ,
  puesto_id INTEGER  NOT NULL  ,
  observaciones VARCHAR(500)    ,
  activo VARCHAR(1)  NOT NULL    ,
PRIMARY KEY(centro_fisico_id, servicio_id, categoria_id, puesto_id)        ,
  FOREIGN KEY(categoria_id)
    REFERENCES unit(id)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(centro_fisico_id)
    REFERENCES unit(id)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(servicio_id)
    REFERENCES unit(id)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(puesto_id)
    REFERENCES unit(id)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION);
CREATE TABLE "contrato_ausencia" (
	`id`	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	`contrato_id`	INTEGER NOT NULL,
	`ausencia_id`	INTEGER NOT NULL,
	`fecha_inicio`	DATE NOT NULL,
	`fecha_fin`	DATE NOT NULL,
	`anno_devengo`	INTEGER NOT NULL,
	`activo`	VARCHAR(1) NOT NULL,
	`ausencia_parcial`	VARCHAR(1) NOT NULL,
	`hora_inicio`	VARCHAR(5),
	`hora_fin`	VARCHAR(5),
	FOREIGN KEY(`contrato_id`) REFERENCES `contrato`(`id`) ON DELETE NO ACTION ON UPDATE NO ACTION,
	FOREIGN KEY(`ausencia_id`) REFERENCES `ausencia`(`id`) ON DELETE NO ACTION ON UPDATE NO ACTION
);
CREATE TABLE "contrato" (
	`id`	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	`cargo_id`	INTEGER NOT NULL,
	`fecha_inicio`	DATE NOT NULL,
	`fecha_fin`	DATE,
	`categoria_profesional_id`	INTEGER NOT NULL,
	`persona_id`	INTEGER NOT NULL,
	FOREIGN KEY(`cargo_id`) REFERENCES `cargo`(`id`) ON DELETE NO ACTION ON UPDATE NO ACTION,
	FOREIGN KEY(`categoria_profesional_id`) REFERENCES `categoria_profesional`(`id`) ON DELETE NO ACTION ON UPDATE NO ACTION,
	FOREIGN KEY(`persona_id`) REFERENCES `persona`(`id`) ON DELETE NO ACTION ON UPDATE NO ACTION
);
CREATE TABLE cobertura_equipo (
  id INTEGER  NOT NULL  ,
  fecha_inicio DATE  NOT NULL  ,
  categoria_id INTEGER  NOT NULL  ,
  lunes INTEGER  NOT NULL  ,
  martes INTEGER  NOT NULL  ,
  miercoles INTEGER  NOT NULL  ,
  jueves INTEGER  NOT NULL  ,
  viernes INTEGER  NOT NULL  ,
  sabado INTEGER  NOT NULL  ,
  domingo INTEGER  NOT NULL  ,
  fecha_fin DATE  NOT NULL    ,
PRIMARY KEY(id)  ,
  FOREIGN KEY(categoria_id)
    REFERENCES unit(id)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION);
CREATE TABLE "ciclo_master" (
	`id`	INTEGER NOT NULL,
	`codigo`	VARCHAR(5) NOT NULL UNIQUE,
	`descripcion`	VARCHAR(255) NOT NULL UNIQUE,
	`cuenta_festivo`	VARCHAR(1) NOT NULL,
	`activo`	VARCHAR(1) NOT NULL,
	PRIMARY KEY(id)
);
CREATE TABLE ciclo_detail (
  ciclo_master_id INTEGER  NOT NULL  ,
  id INTEGER  NOT NULL  ,
  turno_master_id INTEGER  NOT NULL    ,
PRIMARY KEY(ciclo_master_id, id)    ,
  FOREIGN KEY(ciclo_master_id)
    REFERENCES ciclo_master(id)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(turno_master_id)
    REFERENCES turno_master(id)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION);
CREATE TABLE "categoria_profesional" (
	`id`	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	`codigo`	VARCHAR(5) NOT NULL UNIQUE,
	`descripcion`	VARCHAR(255) NOT NULL UNIQUE,
	`activo`	VARCHAR(1) NOT NULL
);
CREATE TABLE "categoria_equipo" (
	`id`	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	`categoria_id`	INTEGER NOT NULL,
	`categoria_profesional_id`	INTEGER NOT NULL,
	FOREIGN KEY(`categoria_id`) REFERENCES `unit`(`id`) ON DELETE NO ACTION ON UPDATE NO ACTION,
	FOREIGN KEY(`categoria_profesional_id`) REFERENCES `categoria_profesional`(`id`) ON DELETE NO ACTION ON UPDATE NO ACTION
);
CREATE TABLE "cargo" (
	`id`	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	`codigo`	VARCHAR(5) NOT NULL UNIQUE,
	`descripcion`	VARCHAR(255) NOT NULL UNIQUE,
	`observaciones`	VARCHAR(500),
	`activo`	VARCHAR(1) NOT NULL
);
CREATE TABLE cambio_turno (
  dia DATE  NOT NULL  ,
  tarea_id INTEGER  NOT NULL  ,
  turno_modificado VARCHAR(5)  NOT NULL  ,
  turno_original VARCHAR(5)  NOT NULL  ,
  observaciones VARCHAR(500)      ,
PRIMARY KEY(dia, tarea_id)  ,
  FOREIGN KEY(tarea_id)
    REFERENCES tarea(id)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION);
CREATE TABLE calendario_festivo (
  centro_fisico_id INTEGER  NOT NULL  ,
  fecha DATE  NOT NULL  ,
  descripcion VARCHAR(255)  NOT NULL  ,
  observaciones VARCHAR(500)      ,
PRIMARY KEY(centro_fisico_id, fecha)  ,
  FOREIGN KEY(centro_fisico_id)
    REFERENCES unit(id)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION);
CREATE TABLE "basica" (
	`id`	INTEGER NOT NULL,
	`version`	VARCHAR(20) NOT NULL,
	`revision`	VARCHAR(20) NOT NULL,
	`nombre`	VARCHAR(20) NOT NULL,
	`descripcion`	VARCHAR(255) NOT NULL,
	`es_lunes_festivo`	INTEGER NOT NULL,
	`es_martes_festivo`	INTEGER NOT NULL,
	`es_miercoles_festivo`	INTEGER NOT NULL,
	`es_jueves_festivo`	INTEGER NOT NULL,
	`es_viernes_festivo`	INTEGER NOT NULL,
	`es_sabado_festivo`	VARCHAR(1) NOT NULL,
	`es_domingo_festivo`	VARCHAR(1) NOT NULL,
	`licencia`	TEXT NOT NULL,
	`empresa`	INTEGER,
	`m_inicio`	VARCHAR(5),
	`m_fin`	VARCHAR(5),
	`t_inicio`	VARCHAR(5),
	`t_fin`	VARCHAR(5),
	`n_inicio`	VARCHAR(5),
	`n_fin`	VARCHAR(5),
	PRIMARY KEY(`id`)
);
CREATE TABLE "ausencia" (
	`id`	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	`codigo`	VARCHAR(5) NOT NULL UNIQUE,
	`descripcion`	VARCHAR(255) NOT NULL UNIQUE,
	`cuenta_horas`	VARCHAR(1) NOT NULL,
	`cuenta_dias`	VARCHAR(1) NOT NULL,
	`max_ausencia_anual`	INTEGER,
	`activar_control_ausencia`	VARCHAR(1) NOT NULL,
	`forzar_ausencia`	VARCHAR(1) NOT NULL,
	`observaciones`	VARCHAR(500),
	`activo`	VARCHAR(1) NOT NULL,
	`estado_devengo`	INTEGER NOT NULL
);
CREATE INDEX usuario_estructura_FKIndex2 ON usuario_estructura (usuario_id);
CREATE INDEX unit_FKIndex1 ON unit (tipo_unit_id)




;
CREATE INDEX turno_detail_FKIndex1 ON turno_detail (turno_master_id);
CREATE INDEX servicios_previos_FKIndex1 ON servicios_previos (persona_id)
;
CREATE INDEX jornada_teorica_FKIndex1 ON jornada_teorica (centro_fisico_id);
CREATE INDEX estructura_FKIndex4 ON estructura (puesto_id);
CREATE INDEX estructura_FKIndex3 ON estructura (categoria_id);
CREATE INDEX estructura_FKIndex2 ON estructura (centro_fisico_id);
CREATE INDEX estructura_FKIndex1 ON estructura (servicio_id);
CREATE INDEX cobertura_equipo_FKIndex1 ON cobertura_equipo (categoria_id);
CREATE INDEX ciclo_detail_FKIndex2 ON ciclo_detail (turno_master_id);
CREATE INDEX ciclo_detail_FKIndex1 ON ciclo_detail (ciclo_master_id);
CREATE INDEX cambio_turno_FKIndex1 ON cambio_turno (tarea_id);
CREATE INDEX calendario_festivo_FKIndex1 ON calendario_festivo (centro_fisico_id);
CREATE VIEW v_planificacion AS 
select dia, turno, anno, mes, categoria_id equipo_id, tarea_id
from (
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 1 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia1 end turno,  mes, categoria_id, anno, tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 1 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-01' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia1
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 2 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia2 end turno, mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 2 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-02' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia2
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 3 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia3 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 3 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-03' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia3
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 4 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia4 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 4 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-04' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia4
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 5 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia5 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 5 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-05' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia5

union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 6 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia6 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 6 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-06' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia6
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 7 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia7 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 7 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-07' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia7
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 8 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia8 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 8 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-08' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia8
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 9 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia9 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and  9 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-09' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia9
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 10 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia10 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 10 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-10' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia10
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 11 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia11 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 11 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-11' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia11
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 12 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia12 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 12 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-12' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia12
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 13 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia13 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 13 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-13' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia13
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 14 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia14 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 14 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-14' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia14
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 15 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia15 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 15 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-15' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia15
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 16 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia16 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 16 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-16' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia16
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 17 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia17 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 17 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-17' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia17
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 18 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia18 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 18 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-18' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia18
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 19 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia19 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 19 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-19' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia19
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 20 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia20 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 20 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-20' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia20
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 21 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia21 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 21 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-21' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia21
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 22 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia22 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 22 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-22' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia22
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 23 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia23 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 23 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-23' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia23
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 24 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia24 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 24 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-24' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia24
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 25 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia25 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 25 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-25' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia25
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 26 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia26 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 26 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-26' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia26
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 27 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia27 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 27 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-27' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia27
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 28 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia28 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 28 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-28' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia28
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 29 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia29 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 29 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-29' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia29
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 30 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia30 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 30 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-30' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia30
union all
select case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 31 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then dia31 end turno,mes, categoria_id,anno,tarea.id tarea_id,
	      case when mes between cast(strftime('%m', tarea.fecha_inicio) as integer) and cast(strftime('%m', tarea.fecha_fin) as integer) and 31 between cast(strftime('%d', tarea.fecha_inicio) as integer) and cast(strftime('%d', tarea.fecha_fin) as integer) then planificacion.anno||'-'||substr('00'||planificacion.mes,length('00'||planificacion.mes) -1)||'-31' end dia
from planificacion join estructura  on estructura.puesto_id = planificacion.puesto_id
                            join tarea on tarea.puesto_id = planificacion.puesto_id
							join turno_master t1 on t1.codigo = planificacion.dia31
) where turno is not null 

-- Se parametriza así: anno = 2017 and mes = 1 and categoria_id = 5 order by dia
/*

select * from v_planificacion where anno = 2017 and mes = 1 and categoria_id = 5

*/;
CREATE VIEW v_diario as
select cf_id, sf_id, eq_id, p_id, persona_id, dni, dia, turno, descripcion,
       case when dif_horas_0_0 is null then dif_horas_0_1 else dif_horas_0_0 end horas,
	   case when dif_min_0_1 is null then dif_min_0_0 else dif_min_0_1 end minutos
  from (
select p.cf_id, p.sf_id, p.eq_id, p.p_id,
       persona_id, dni, dia, turno, turno_master.descripcion, turno_detail.dia_inicial, turno_detail.dia_final, turno_detail.hora_inicio, turno_detail.hora_fin,
case when turno_detail.dia_inicial = turno_detail.dia_final then 

cast(((strftime('%s', '2017-01-01 '||turno_detail.hora_fin) - strftime('%s', '2017-01-01 '||turno_detail.hora_inicio)) % (60 * 60 * 24)) / (60 * 60) as integer)
else null end dif_horas_0_0,
case when turno_detail.dia_inicial = turno_detail.dia_final then 
cast(((strftime('%s', '2017-01-01 '||turno_detail.hora_fin) - strftime('%s', '2017-01-01 '||turno_detail.hora_inicio)) % (60 * 60 * 24)) % (60 * 60) / 60 as integer)
else null end dif_min_0_0,

case when turno_detail.dia_inicial = 0 and turno_detail.dia_final = 1 then 
(cast(((strftime('%s', '2017-01-01 24:00') - strftime('%s', '2017-01-01 '||turno_detail.hora_inicio)) % (60 * 60 * 24)) / (60 * 60) as integer) + 
cast(((strftime('%s', '2017-01-01 '||turno_detail.hora_fin) - strftime('%s', '2017-01-01 00:00')) % (60 * 60 * 24)) / (60 * 60) as integer))
else null end dif_horas_0_1,

case when turno_detail.dia_inicial = 0 and turno_detail.dia_final = 1 then 
(cast(((strftime('%s', '2017-01-01 24:00') - strftime('%s', '2017-01-01 '||turno_detail.hora_inicio)) % (60 * 60 * 24)) % (60 * 60) / 60 as integer) + 
cast(((strftime('%s', '2017-01-01 '||turno_detail.hora_fin) - strftime('%s', '2017-01-01 00:00')) % (60 * 60 * 24)) % (60 * 60) / 60 as integer))
else null end dif_min_0_1 
from (
select v_planificacion.dia, coalesce(cambio_turno.turno_modificado, v_planificacion.turno) turno, persona.id persona_id, persona.dni,
       estructura.centro_fisico_id cf_id, estructura.servicio_id sf_id, estructura.categoria_id eq_id, estructura.puesto_id p_id
  from tarea join contrato on contrato.id = tarea.contrato_id
             join persona on persona.id = contrato.persona_id
			 join estructura on estructura.puesto_id = tarea.puesto_id
			 join v_planificacion on v_planificacion.equipo_id = estructura.categoria_id
			      and v_planificacion.tarea_id = tarea.id
	    left join cambio_turno on cambio_turno.tarea_id = v_planificacion.tarea_id
	              and cambio_turno.dia = v_planificacion.dia   				  

) p join turno_master on turno_master.codigo = p.turno
    join turno_detail on turno_detail.turno_master_id = turno_master.id
) group by persona_id, dni, dia, turno, descripcion, cf_id, sf_id, eq_id, p_id;
CREATE VIEW v_balance AS
select cf_id,
	       sf_id,
	       eq_id,
	       p_id, 
       dia, 
       persona_id, 
	   dni, 
	   nombre, 
	   ape1, 
	   ape2, 
	   sp, 
	   horas_turno - horas_ausencia horas_presencia, 
	   anno_ausencia, 
	   devengo_ausencia, 
	   estado_devengo
  from (
select v_diario.dia,
       v_diario.cf_id,
	   v_diario.sf_id,
	   v_diario.eq_id,
	   v_diario.p_id,
	   persona.id persona_id,
       persona.dni,
       persona.nombre,
       persona.ape1,
       persona.ape2,
	   coalesce(0,servicios_previos.horas) sp,
	   cast(v_diario.horas as float) + (cast(v_diario.minutos as float) / 60) horas_turno,
	   case when v_ausencia.aus_cuenta_horas = 'S' or v_ausencia.aus_cuenta_horas is null then 0
	        when v_ausencia.aus_cuenta_horas = 'N' then v_ausencia.horas_ausencia end horas_ausencia,
	   v_ausencia.anno anno_ausencia,
	   v_ausencia.anno_devengo devengo_ausencia,
	   v_ausencia.estado_devengo
  from v_diario join persona on persona.id = v_diario.persona_id
                left join servicios_previos on servicios_previos.persona_id = v_diario.persona_id
                left join v_ausencia on v_ausencia.persona_id = v_diario.persona_id
                          and v_ausencia.fecha = v_diario.dia
);
CREATE VIEW v_ausencia AS
select anno,
       anno_devengo,
	   estado_devengo,
       case when horas_turno = 0 then 0.0 
	        when horas_ausencia >= horas_turno then horas_turno
			else horas_ausencia end horas_ausencia,
	   cod_aus,
       desc_aus,
       fini_aus,
	   ffin_aus,
       ausencia_parcial,
       hora_inicio,
       hora_fin,
       aus_id,
       aus_cuenta_horas,
       aus_cuenta_dias,
       turno,
	   horas_turno,
       fecha,
	   persona_id,
       tarea_id,
       contrato_id
  from (
select strftime('%Y', contrato_ausencia.fecha_inicio) anno, 
       contrato_ausencia.anno_devengo anno_devengo,            
       ausencia.estado_devengo,
       case when contrato_ausencia.ausencia_parcial = 'S' then cast((strftime('%s', '2017-01-01 '||contrato_ausencia.hora_fin) - strftime('%s', '2017-01-01 '||contrato_ausencia.hora_inicio)) as float)
            when contrato_ausencia.ausencia_parcial = 'N' then cast((v_diario.horas * 60 * 60) + (v_diario.minutos * 60) as float) end / 3600 horas_ausencia,
	   cast((v_diario.horas * 60 * 60) + (v_diario.minutos * 60) as float) / 3600 horas_turno,
       contrato_ausencia.ausencia_parcial,
       contrato_ausencia.hora_inicio,
       contrato_ausencia.hora_fin,
       tarea.id tarea_id, 
       contrato.id contrato_id,
       v_diario.turno, 
       v_diario.dia fecha,
       contrato_ausencia.fecha_inicio fini_aus,
	   contrato_ausencia.fecha_fin ffin_aus,
       persona.id persona_id,
       ausencia.id aus_id,
       ausencia.codigo cod_aus,
       ausencia.descripcion desc_aus,
       ausencia.cuenta_horas aus_cuenta_horas,
       ausencia.cuenta_dias aus_cuenta_dias
  from estructura join tarea on tarea.puesto_id = estructura.puesto_id
                  join contrato on contrato.id = tarea.contrato_id
                  join persona on persona.id = contrato.persona_id
                  join v_diario on v_diario.persona_id = contrato.persona_id 
                  join turno_master on turno_master.codigo = v_diario.turno
                  join contrato_ausencia on contrato_ausencia.contrato_id = contrato.id
                       and v_diario.dia between contrato_ausencia.fecha_inicio and contrato_ausencia.fecha_fin
                  join ausencia on ausencia.id = contrato_ausencia.ausencia_id
);
COMMIT;
